import sys
import os

from testindatadev.s3.minio.minio import YcMinio
from testindatadev.s3.qiniu.qiniu import Qiniu
from testindatadev.dataset.request import Request
from testindatadev.dataset.updatefile import UpdateFile

class Dataset():
    def __init__(self, T_key, datasetName, host):
        self.OSS_TYPE_DEFAULT = 0
        self.OSS_TYPE_MINIO = 0
        self.OSS_TYPE_ALI = 1
        self.OSS_TYPE_AWS = 2
        self.OSS_TYPE_QINIU = 3

        self.DATASET_TYPE_IMAGE = 0
        self.DATASET_TYPE_VIDEO = 1
        self.DATASET_TYPE_AUDIO = 2
        self.DATASET_TYPE_POINT_CLOUD = 3
        self.DATASET_TYPE_FUSION_POINT_CLOUD = 4
        self.DATASET_TYPE_POINT_CLOUD_SEMANTIC_SEGMENTATION = 5
        self.DATASET_TYPE_TEXT = 6

        #授权云存储
        self.HOSTING_METHOD_CLOUD = 1
        #自由云存储
        self.HOSTING_METHOD_OWN_STORAGE = 2

        self.req = Request(T_key, datasetName, host)
        info = self.req.GetAccess()

        # self.access_key = info['access_key']
        # self.secret_key = info['secret_key']
        self.upload_token = info['upload_token']
        self.endpoint = info['endpoint']

        self.bucket = info['bucket']
        self.file_path = info["file_path"]
        self.oss_type = info['oss_type']
        self.datasetName = datasetName
        self.datasetType = info["dataset_type"]
        self.hostingMethod = info["hosting_method"]
        self.host = host

        #非实际云存储
        if self.hostingMethod != self.HOSTING_METHOD_OWN_STORAGE:
            if self.oss_type == self.OSS_TYPE_MINIO or \
                self.oss_type == self.OSS_TYPE_DEFAULT or \
                self.oss_type == self.OSS_TYPE_ALI or \
                self.oss_type == self.OSS_TYPE_AWS:
                # self.client = YcMinio(self.access_key, self.secret_key, self.endpoint)
                # if self.bucket not in self.client.ListAllBucket():
                #     raise Exception(f"bucket:{self.bucket} is not exist!")
                self.client = YcMinio(self.req)
            elif self.oss_type == self.OSS_TYPE_QINIU:
                self.client = Qiniu(self.upload_token)
            else:
                raise Exception("尚未支持其他类型的云存储")
        else:
            self.client = None

    def PutFileToDataset(self, objectName, filePath):
        if self.oss_type == self.OSS_TYPE_QINIU:
            self.client.PutObject(self.bucket, self.datasetName + "/" + objectName, filePath)
        else:
            self.client.PutObject(self.bucket, objectName, filePath)

        # if not self.file_path:
        #     self.client.PutObject(self.bucket, self.datasetName + "/" + objectName, filePath)
        # else:
        #     self.client.PutObject(self.bucket, self.file_path + "/" + self.datasetName + "/" + objectName, filePath)

    def SyncDataToWeb(self, data):
        info = self.req.Upload(data)
        return info

    def GetData(self, offset, limit, metaData={}, label="", sensor="", debug=False):
        info = self.req.GetData(offset, limit, metaData, label, sensor)
        if "total" not in info.keys() or "files" not in info.keys():
            return {"total":0,"files":[]}

        ret = {
            "total":info["total"],
            "files":[]
        }

        for item in info["files"]:
            file = UpdateFile(item, self.req, self.datasetType, debug=debug)
            ret["files"].append(file)

        return ret

    def GetFileAndLabelByFid(self, fid, debug=False):
        info = self.req.GetFileAndLabelByFid(fid)
        if "file" not in info.keys() or info["file"] == {}:
            return

        file = UpdateFile(info["file"], self.req, self.datasetType, debug=debug)
        file.anotations.LoadFromList(info["anotations"])
        return file

    def GetFileAndLabelByReferid(self, referId, debug=False):
        info = self.req.GetFileAndLabelByReferid(referId)
        if "file" not in info.keys() or info["file"] == {}:
            return

        file = UpdateFile(info["file"], self.req, self.datasetType, debug=debug)
        file.anotations.LoadFromList(info["anotations"])
        return file




from testindatadev.dataset.file import File
from testindatadev.dataset.labeldata import LabelData
import json
import copy

class UpdateFile(File):
    def __init__(self, fdata={}, req=None, fileType="", debug=False):
        # do not edit this file, magic!!!!
        if not req:
            raise Exception(f"request for FileData must be set")

        keys = list(set(fdata.keys()))
        keys.sort()
        if keys != ['create_at', 'fid', 'fid_num', 'frame_id', 'md5', 'meta', 'name', 'path', 'ref_id', 'sensor', 'size', 'update_at', 'url']:
            raise Exception(f"bad fdata parameter, keys:{keys}")

        meta = fdata["meta"]
        if meta == []:
            meta = {}

        self.__oldData = {
            "meta": copy.copy(meta),
            "name": copy.copy(fdata["name"]),
            "frame_id": copy.copy(fdata["frame_id"]),
            "sensor": copy.copy(fdata["sensor"]),
            "ref_id": copy.copy(fdata["ref_id"])
        }

        self.__fid = fdata["fid"]
        self.__fidNum = fdata["fid_num"]
        self.referId = fdata["ref_id"]
        self.name = fdata["name"]
        self.__path = fdata["path"]
        self.__url = fdata["url"]
        self.__md5 = fdata["md5"]
        self.__size = fdata["size"]
        self.frameId = fdata["frame_id"]
        self.sensor = fdata["sensor"]
        self.metaData = meta
        self.__createAt = fdata["create_at"]
        self.__updateAt = fdata["update_at"]

        self.__list = fdata

        self.__req = req
        self.labeldata = LabelData()
        self.anotations = LabelData()

        self.TYPE_IMAGE = 0
        self.TYPE_VIDEO = 1
        self.TYPE_AUDIO = 2
        self.TYPE_POINT_CLOUD = 3
        self.TYPE_FUSION_POINT_CLOUD = 4
        self.TYPE_POINT_CLOUD_SEMANTIC_SEGMENTATION = 5
        self.TYPE_TEXT = 6

        if fileType == "":
            self.fileType = fileType
        else:
            self.fileType = self.TYPE_IMAGE

        self.debug = debug

    @property
    def path(self):
        return self.__path

    @property
    def req(self):
        return self.__req

    @property
    def fid(self):
        return self.__fid

    @property
    def fidNum(self):
        return self.__fidNum

    @property
    def url(self):
        return self.__url

    @property
    def md5(self):
        return self.__md5

    @property
    def size(self):
        return self.__size

    @property
    def createAt(self):
        return self.__createAt

    @property
    def updateAt(self):
        return self.__updateAt

    def ToList(self):
        ret = {}
        ret["file"] = self.__list
        if len(self.anotations.labels) > 0:
            ret["anotations"] = self.anotations.labels

        return ret

    def Update(self):
        self.SelfCheck()

        updateFlag = False

        updateMeta = {}
        if self.metaData != self.__oldData["meta"]:
            updateMeta = self.metaData
            updateFlag = True
            if self.debug:
                print(f"[UPDATE_FILE] fid: {self.fid}, metaData:{updateMeta}")

        updateName = ""
        if self.name != self.__oldData["name"]:
            updateName = str(self.name)
            updateFlag = True
            if self.debug:
                print(f"[UPDATE_FILE] fid: {self.fid}, name:{self.name}")

        updateFrameId = ""
        if self.frameId != self.__oldData["frame_id"]:
            updateFrameId = str(self.frameId)
            updateFlag = True
            if self.debug:
                print(f"[UPDATE_FILE] fid: {self.fid}, frameId:{self.frameId}")

        updateSensor = ""
        if self.sensor != self.__oldData["sensor"]:
            updateSensor = str(self.sensor)
            updateFlag = True
            if self.debug:
                print(f"[UPDATE_FILE] fid: {self.fid}, sensor:{self.sensor}")

        updateReferId = ""
        if self.referId != self.__oldData["ref_id"]:
            updateReferId = str(self.referId)
            updateFlag = True
            if self.debug:
                print(f"[UPDATE_FILE] fid: {self.fid}, referId:{self.referId}")

        if self.debug:
            if len(self.labeldata.labels) > 0:
                updateFlag = True
                print(f"[UPDATE_FILE] fid: {self.fid}, update label data!!")

        if not updateFlag:
            if self.debug:
                print(f"[UPDATE_FILE] fid: {self.fid}, there is nothing different from old data, abort!")
            return

        data = {
            "meta":updateMeta,
            "name":updateName,
            "frame_id":updateFrameId,
            "sensor":updateSensor,
            "anotations":self.labeldata.labels,
            "ref_id":updateReferId,
        }

        return self.req.Update(self.fid, data)

    # def GetLabels(self):
    #     info = copy.deepcopy(self.labeldata.labels)
    #     return info
    #
    # def DeleteLabels(self):
    #     self.labeldata.labels = []
    #     return True

    def Delete(self):
        return self.req.Delete(self.fid)





import requests
import json

class Request:
    def __init__(self, T_key, dataset_id, host):
        self.T_key = T_key
        self.dataset_id = dataset_id
        self.host = ""
        if host:
            self.host = host.rstrip("/")
            self.API_ADDRESS_PREFIX = f"{self.host}/api/v1/"
        else:
            #todo:设置线上服务地址
            raise Exception("the host to the testin-dataset server must be set!")

    def RiseException(self, response_text):
        raise Exception("error msg from server: " + response_text)

    def GetAccess(self):
        api = f'{self.API_ADDRESS_PREFIX}dataset/{self.dataset_id}'
        headers = {"T-Key":self.T_key}
        res = requests.request("GET", api, headers=headers)
        try:
            resData = json.loads(res.text)
        except:
            raise Exception(res.content)


        if resData['code'] != "":
            self.RiseException(res.text)

        ret = {
            # "access_key":resData["data"]["access_key"],
            # "secret_key":resData["data"]["access_token"],
            "hosting_method":resData["data"]["hosting_method"],
            "upload_token":"",
            "endpoint":resData["data"]["endpoint"],
            "file_path": "",
            "bucket":resData["data"]["bucket"],
            "oss_type":resData["data"]["oss_type"],
            "dataset_type":resData["data"]["dataset_type"],
        }

        if "upload_token" in resData["data"].keys():
            ret["upload_token"] = resData["data"]["upload_token"]

        return ret

    def GetUploadUrl(self, object):
        api = f'{self.API_ADDRESS_PREFIX}dataset/{self.dataset_id}/get-upload-url'
        headers = {"T-Key": self.T_key, "Content-Type":"application/json"}

        data1 = {
            "object":object
        }

        res = requests.post(api, headers=headers, data=json.dumps(data1))
        try:
            resData = json.loads(res.text)
        except:
            raise Exception(res.content)

        if resData['code'] != "":
            self.RiseException(res.text)

        ret = {
            "url":resData["data"]["url"]
        }

        return ret


    def Upload(self, data):
        api = f'{self.API_ADDRESS_PREFIX}dataset/{self.dataset_id}/upload'
        headers = {"T-Key": self.T_key, "Content-Type":"application/json"}

        res = requests.post(api, headers=headers, data=json.dumps(data))
        try:
            resData = json.loads(res.text)
        except:
            raise Exception(res.content)

        if resData['code'] != "":
            self.RiseException(res.text)

        ret = {
            "succ": resData["data"]["succ"],
            "fail": resData["data"]["fail"],
        }

        return ret

    def Delete(self, referId):
        api = f'{self.API_ADDRESS_PREFIX}dataset/{self.dataset_id}/fidloc/{referId}'
        headers = {"T-Key": self.T_key}
        res = requests.delete(api, headers=headers)

        try:
            resData = json.loads(res.text)
        except:
            raise Exception(res.content)

        if resData['code'] != "":
            self.RiseException(res.text)

        ret = {
            "file_num": resData["data"]["file_num"],
            "anotation_num": resData["data"]["anotation_num"],
        }

        return ret

    def GetData(self, offset, limit, metaData={}, label="", sensor=""):
        api = f'{self.API_ADDRESS_PREFIX}dataset/{self.dataset_id}/data'
        headers = {"T-Key": self.T_key, "Content-Type": "application/json"}
        data = {
            "offset":offset,
            "limit":limit,
            "meta":metaData,
            "label":label,
            "sensor":sensor,
        }

        res = requests.post(api, headers=headers, data=json.dumps(data))
        try:
            resData = json.loads(res.text)
        except:
            raise Exception(res.content)

        if resData['code'] != "":
            self.RiseException(res.text)


        return resData["data"]

    def GetFileAndLabelByFid(self, fid):
        api = f'{self.API_ADDRESS_PREFIX}dataset/{self.dataset_id}/fidloc/{fid}'
        headers = {"T-Key": self.T_key}
        res = requests.request("GET", api, headers=headers)
        try:
            resData = json.loads(res.text)
        except:
            raise Exception(res.content)

        if resData['code'] != "":
            self.RiseException(res.text)

        return resData["data"]


    def GetFileAndLabelByReferid(self, referId):
        api = f'{self.API_ADDRESS_PREFIX}dataset/{self.dataset_id}/refloc/{referId}'
        headers = {"T-Key": self.T_key}
        res = requests.request("GET", api, headers=headers)
        try:
            resData = json.loads(res.text)
        except:
            raise Exception(res.content)

        if resData['code'] != "":
            self.RiseException(res.text)

        return resData["data"]

    def Update(self, fid, data={}):
        api = f'{self.API_ADDRESS_PREFIX}dataset/{self.dataset_id}/fidloc/{fid}/update'
        headers = {"T-Key": self.T_key, "Content-Type": "application/json"}

        res = requests.request("POST", api, headers=headers, data=json.dumps(data))
        try:
            resData = json.loads(res.text)
        except:
            raise Exception(res.content)

        if resData['code'] != "":
            self.RiseException(res.text)

        return resData["data"]



import json

class MetaData():
    def __init__(self, metadata):
        # 数据格式检查
        if not type(metadata) is dict:
            raise Exception(f"metadata must be a dict, {type(metadata)} gavin")
        self.meta = metadata

    def ToString(self):
        return json.dumps(self.meta)
